
#pragma once

#include <Base_Monitor.h>
#include <Base_Configurator.h>
#include <chrono>
#include <unordered_map>

/**
 * @brief Namespace containing packet monitors for the ASTRI DAQ system.
 */
namespace inaf::oasbo::PacketMonitors {

/**
 * @brief Class representing an ASTRI packet monitor.
 * 
 * This class inherits from the BasePacketMonitor class and provides functionality
 * to monitor ASTRI packets, calculate statistics, and print the statistics.
 */
class AstriPacketMonitor: public BasePacketMonitor {

protected:
	std::chrono::duration<double, std::milli> timeElps; /**< Elapsed time since monitoring started. */
	std::chrono::steady_clock::time_point startTime; /**< Start time of monitoring. */
	std::unordered_map<std::string, size_t> lastSSCMap; /**< Map of last sequence numbers for each packet type. */
	std::unordered_map<std::string, size_t> lostsMap; /**< Map of lost packets for each packet type. */
	std::chrono::steady_clock::time_point lastNsTime; /**< helper variable. */
	double bitsLastNs; /**< Number of bits received in the last N seconds. */
	size_t packsLastNs; /**< Number of packets received in the last N seconds. */
	uint update_interval; /**< Interval at which statistics are updated. */

	/**
	 * @brief Monitors a packet for loss and updates the lost packets map.
	 * 
	 * @param pack The packet to monitor.
	 */
	void monitPacketLost(PacketLib::BasePacket &pack);

	static std::string tot_packets_key; /**< Key for total packets statistic. */
	static std::string tot_MB_key; /**< Key for total MB statistic. */
	static std::string tot_lost_key; /**< Key for total lost packets statistic. */
	static std::string byte_rate_key; /**< Key for byte rate statistic. */
	static std::string mbit_rate_key; /**< Key for megabit rate statistic. */
	static std::string packet_rate_key; /**< Key for packet rate statistic. */

	friend class AstriPacketMonitorBuilder;

public:
	/**
	 * @brief Sets the update interval for statistics.
	 * 
	 * @param interval The update interval in milliseconds.
	 */
	void setUpdateInterval(int interval);

	/**
	 * @brief Gets the update interval for statistics.
	 * 
	 * @return The update interval in milliseconds.
	 */
	uint getUpdateInterval(){return update_interval;}

	/**
	 * @brief Monitors a packet and updates the statistics.
	 * 
	 * @param pack The packet to monitor.
	 */
	void monit(PacketLib::BasePacket &pack);

	/**
	 * @brief Resets the monitor and clears the statistics.
	 */
	void reset();

	/**
	 * @brief Prints the current statistics.
	 */
	void printStats();
};

/**
 * @brief Builder class for creating an AstriPacketMonitor object.
 */
class AstriPacketMonitorBuilder {
protected:
	AstriPacketMonitor *monitor; /**< Pointer to the AstriPacketMonitor object being built. */

public:
	std::string config_target { "astrisimplemonitor" }; /**< Configuration target for the monitor. */
	std::string interval_key { "rateinterval" }; /**< Key for the update interval configuration. */

	/**
	 * @brief Default constructor.
	 */
	AstriPacketMonitorBuilder();

	/**
	 * @brief Destructor.
	 */
	~AstriPacketMonitorBuilder();

	/**
	 * @brief Resets the builder to its initial state.
	 */
	void reset();

	/**
	 * @brief Configures the monitor from a BaseConfigurator object.
	 * 
	 * @param conf The BaseConfigurator object to configure from.
	 * @return A pointer to the AstriPacketMonitorBuilder object.
	 */
	AstriPacketMonitorBuilder* configFrom(Configurators::BaseConfigurator &conf);

	/**
	 * @brief Sets the update interval for the monitor.
	 * 
	 * @param interval The update interval in seconds.
	 * @return A pointer to the AstriPacketMonitorBuilder object.
	 */
	AstriPacketMonitorBuilder* setUpdateInterval(int interval);

	/**
	 * @brief Gets the built AstriPacketMonitor object.
	 * 
	 * @return A pointer to the AstriPacketMonitor object.
	 */
	AstriPacketMonitor* getMonitor();
};

} 